package binarization;

import binarization.data.IBinaryImage;
import java.text.Format;
import java.util.Comparator;

/**
 * Interface que representa uma metrica para comparar duas imagens binarias,
 * devolvendo um valor cujo significado exato depende da metrica.
 * 
 * <p>Nenhuma metrica deveria devolver NaN ou INFINITY.</p>
 * 
 * <p>Espera-se que cada objeto de uma classe de metrica seja imutavel.
 * (para poder ser usado como chave num {@link java.util.Map})</p>
 *
 * @author denis
 * @see IBinaryImage
 * @see IBinarizationMethod
 */
public interface IBinaryImageMetric {

	/**
	 * Devolve uma avaliacao da imagem binarizada em relacao ao <em>ground truth</em>.
	 * 
	 * <p>O significado desta avaliacao depende da metrica que implementa este metodo.</p>
	 * 
	 * <p>Este metodo e considerado o padrao. Espera-se que a grande maioria das
	 * metricas implemente este metodo. Para estas metricas,
	 * a seguinte igualdade deve valer:</p>
	 * 
	 * <p>{@code evaluate(GT, img) == evaluate(GT, img, skeletonized)}</p>
	 *
	 * @param groundTruth a imagem de comparacao
	 * @param binarized a imagem que vai ser avaliada
	 * @return uma avaliacao da imagem binarizada em relacao ao ground truth
	 * @throws IllegalArgumentException se as imagens tiverem tamanhos diferentes
	 */
	double evaluate(IBinaryImage groundTruth, IBinaryImage binarized);

	/**
	 * Devolve uma avaliacao da imagem binarizada em relacao ao <em>ground truth</em>
	 * e ao <em>skeletonized ground truth</em>. Esta ultima imagem e a silhueta
	 * de um pixel de largura que define os objetos do <em>ground truth</em>.
	 * 
	 * <p>O significado desta avaliacao depende da metrica que implementa este metodo.</p>
	 * 
	 * <p>Este metodo nao e obrigatorio. Se uma metrica nao utiliza o terceiro argumento,
	 * ela deve desconsidera-lo. No entanto, se uma metrica necessita deste terceiro
	 * argumento, entao ela deve devolver um valor padrao unico caso uma imagem
	 * vazia seja dado como terceiro argumento, pois pode ser o caso de o
	 * conjunto de imagens fornecido nao possuir o <em>skeletonized ground truth</em>.</p>
	 *
	 * @param groundTruth a imagem de comparacao
	 * @param binarized a imagem que vai ser avaliada
	 * @param skeletonizedGT a silhueta de um pixel de largura da imagem de comparacao
	 * @return uma avaliacao da imagem binarizada em relacao ao ground truth
	 * @throws IllegalArgumentException se as imagens tiverem tamanhos diferentes
	 */
	double evaluate(IBinaryImage groundTruth, IBinaryImage binarized, IBinaryImage skeletonizedGT);

	/**
	 * Devolve um rotulo curto que identifica a metrica.
	 *
	 * @return um rotulo curto que identifica a metrica
	 */
	String getLabel();

	/**
	 * Devolve a formatacao adequada para a avalicao da metrica.
	 *
	 * @return a formatacao adequada para a avalicao da metrica
	 */
	Format getFormat();

	/**
	 * Devolve um comparador entre resultados desta metrica.
	 * 
	 * <p>Em outras palavras, dados {@code r1, r2} dois resultados distintos
	 * de uma metrica, {@code r1} e melhor que {@code r2} se
	 * {@link java.util.Comparator#compare} {@code (r1, r2)} devolve um valor
	 * positivo.</p>
	 * 
	 * @return um comparador entre resultados desta metrica
	 */
	Comparator<Double> getResultComparator();
}
