package binarization;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Classe auxiliar que encapsula a estrutura complexa do resultado.
 * 
 * <p>Nao e imutavel.</p>
 * 
 * @author denis
 * @see Ranking
 * @see IBinarizationMethod
 * @see IBinaryImageMetric
 */
public class Results {

	private final Map<IBinaryImageMetric, Map<IBinarizationMethod, ? extends List<Double>>> map;

	/**
	 * Constroi um acumulador dos resultados das metricas para cada metodo.
	 * 
	 * @param methods os metodos utilizados
	 * @param metrics as metricas utilizadas
	 */
	public Results(Collection<IBinarizationMethod> methods, Collection<IBinaryImageMetric> metrics) {

		map = new HashMap<IBinaryImageMetric, Map<IBinarizationMethod, ? extends List<Double>>>(metrics.size());

		for (IBinaryImageMetric metric : metrics) {
			final Map<IBinarizationMethod, List<Double>> inner = new HashMap<IBinarizationMethod, List<Double>>();

			for (IBinarizationMethod method : methods) {
				inner.put(method, new LinkedList<Double>());
			}

			map.put(metric, inner);
		}
	}

	/**
	 * Adiciona o resultado dado dos metodo e metrica dados aos resultados anteriores
	 * dos mesmos metodo e metrica.
	 * 
	 * @param method o metodo do resultado
	 * @param metric a metrica do resultado
	 * @param result o resultado da metrica para o metodo
	 */
	public void addResult(IBinarizationMethod method, IBinaryImageMetric metric, double result) {
		map.get(metric).get(method).add(result);
	}

	/**
	 * Devolve a media dos resultados da metrica para o metodo.
	 * 
	 * @param method o metodo do resultado
	 * @param metric a metrica do resultado
	 * @return a media dos resultados
	 */
	public double getAverage(IBinarizationMethod method, IBinaryImageMetric metric) {
		final List<Double> results = map.get(metric).get(method);

		double sum = 0.0;
		for (Double d : results) {
			sum += d.doubleValue();
		}

		return sum / results.size();
	}

	/**
	 * Devolve uma lista imutavel com os resultados da metrica para o metodo.
	 * 
	 * @param method o metodo do resultado
	 * @param metric a metrica do resultado
	 * @return uma lista imutavel com os resultados da metrica para o metodo
	 */
	public List<Double> getResults(IBinarizationMethod method, IBinaryImageMetric metric) {
		return Collections.unmodifiableList(map.get(metric).get(method));
	}
}
