package binarization.data;

/**
 * Tipos de contraste utilizados para formar imagens e mapas.
 *
 * @author denis
 */
public enum ContrastType {

	/** Contraste de maximo. Detecta texto.*/
	MAX_ONLY("MaxOnly") {

		@Override
		public double contrastAt(IGrayLevelImage input, ImageSize windowSize, ImagePoint center) {
			final IGrayLevelImage window = input.windowOf(center, windowSize);

			final int max = window.getMaximum();

			return (max - input.pixelAt(center).value()) / (max + Double.MIN_NORMAL);
		}
	},
	/** Contraste de maximo e minimo. Detecta bordas de texto.*/
	MAX_MIN("MaxMin") {

		@Override
		public double contrastAt(IGrayLevelImage input, ImageSize windowSize, ImagePoint center) {
			final IGrayLevelImage window = input.windowOf(center, windowSize);

			final int max = window.getMaximum();
			final int min = window.getMinimum();

			return (max - min) / (max + min + Double.MIN_NORMAL);
		}
	};
	private final String label;

	private ContrastType(String label) {
		this.label = label;
	}

	/**
	 * Devolve o valor de contraste relativo ao ponto dado, utilizando uma
	 * janela em torno dele de tamanho dado.
	 * 
	 * @param input a imagem de entrada
	 * @param windowSize o tamanho da janela para o calculo do contraste
	 * @param center o ponto central da janela
	 * @return o valor de contraste relativo ao ponto dado
	 */
	public abstract double contrastAt(IGrayLevelImage input, ImageSize windowSize, ImagePoint center);

	/**
	 * Devolve o pixel de contraste relativo ao ponto dado, utilizando uma
	 * janela em torno dele de tamanho dado.
	 * 
	 * @param input a imagem de entrada
	 * @param windowSize o tamanho da janela para o calculo do contraste
	 * @param center o ponto central da janela
	 * @return o pixel de contraste relativo ao ponto dado
	 */
	public Pixel contrastPixelAt(IGrayLevelImage input, ImageSize windowSize, ImagePoint center) {
		final double inverted = 1.0 - contrastAt(input, windowSize, center);
		final int intensity = (int) Math.round(Pixel.ofMaxGrayLevel().value() * inverted);

		return Pixel.of(intensity);
	}

	@Override
	public String toString() {
		return label;
	}
}
