package binarization.data;

/**
 * Interface que define uma imagem em niveis de cinza.
 * A intensidade minima e implicitamente zero.
 *
 * @author denis
 * @see IImage
 * @see IBinaryImage
 * @see IHistogram
 * @see Images
 * @see AbstractGrayLevelImage
 * @see Pixel
 * @see ImageSize
 */
public interface IGrayLevelImage extends IImage {

	@Override
	boolean isEmpty();

	@Override
	ImageSize size();

	@Override
	Pixel pixelAt(ImagePoint p);

	@Override
	Pixel pixelAtUnchecked(int x, int y);

	@Override
	IGrayLevelImage windowOf(ImagePoint center, ImageSize size);

	/**
	 * Devolve a inversa desta imagem. Em outras palavras, os pixels p desta
	 * imagem sao trocados por pixels de valor 
	 * {@link Pixel#ofMaxGrayLevel} {@code - p}.
	 * 
	 * @return a inversa desta imagem
	 */
	IGrayLevelImage inverse();

	/**
	 * Devolve o histograma das frequencias das intensidades desta imagem.
	 * 
	 * @return o histograma das frequencias das intensidades desta imagem
	 */
	IHistogram getHistogram();

	int getMinimum();

	int getMaximum();

	/**
	 * Devolve a media das intensidades desta imagem.
	 * 
	 * @return a media das intensidades desta imagem
	 */
	double getMean();

	/**
	 * Devolve o desvio padrao das intensidades desta imagem.
	 * 
	 * @return o desvio padrao das intensidades desta imagem
	 */
	double getStandardDeviation();

	/**
	 * Construtor de uma imagem em niveis de cinza. Cada pixel da imagem 
	 * e preenchido um a um, por fim construindo a imagem final imutavel.
	 * 
	 * <p>Os valores dos pixels deve respeitar a faixa de intensidades definida
	 * na interface {@link GrayLevelImage}.</p>
	 * 
	 * <p>Os pixels que nao forem colocados serao inicializados como
	 * {@link Pixel#ofWhite}.</p>
	 */
	interface Builder {

		/**
		 * Coloca o pixel dado no ponto dado.
		 * 
		 * @param p o ponto onde sera colocado o pixel dado
		 * @param pixel o pixel a ser colocado
		 * @throws IndexOutOfBoundsException se o ponto estiver fora da imagem
		 */
		void setPixel(ImagePoint p, Pixel pixel);

		/**
		 * Coloca o pixel (x, y) dado no ponto dado.
		 * Nao checa se o pixel esta na imagem.
		 * 
		 * @param x a coordenada horizontal do ponto onde sera colocado o pixel dado
		 * @param y a coordenada vertical do ponto onde sera colocado o pixel dado
		 * @param pixel o pixel a ser colocado
		 */
		void setPixelUnchecked(int x, int y, Pixel pixel);

		/**
		 * Constroi a imagem em niveis de cinza com os pixels dados.
		 * Os valores que nao forem colocados possuem valor {@link Pixel#ofWhite}.
		 * 
		 * @return a imagem em niveis de cinza com os pixels dados
		 */
		IGrayLevelImage build();
	}
}
