package binarization.data;

/**
 * Uma imagem em niveis de cinza que possui uma mascara dada por uma imagem
 * binaria de mesmo tamanho. Somente os pixels da imagem em niveis de cinza
 * que correspondem a um pixel preto da imagem binaria serao usados nos
 * calculos de media e desvio padrao.
 * 
 * <p>Embora possua interface semelhante a {@link IGrayLevelImage}, esta
 * imagem nao a implementa, pois algumas operacoes das suas operacoes
 * nao estao bem definidas, como inversa e histograma.</p>
 *
 * @author denis
 */
public class MaskedImage implements IImage {

	private final IGrayLevelImage original;
	private final IBinaryImage mask;
	private Integer cachedMasked;
	private Double cachedMean;
	private Double cachedStd;

	private MaskedImage(IGrayLevelImage original, IBinaryImage mask) {
		this.original = original;
		this.mask = mask;
		this.cachedMasked = null;
		this.cachedMean = null;
		this.cachedStd = null;

		if (!original.size().equals(mask.size())) {
			throw new IllegalArgumentException("Images must be of same size.");
		}
	}

	public static MaskedImage of(IGrayLevelImage original, IBinaryImage mask) {
		return new MaskedImage(original, mask);
	}

	@Override
	public boolean isEmpty() {
		return original.isEmpty();
	}

	@Override
	public ImageSize size() {
		return original.size();
	}

	@Override
	public Pixel pixelAt(ImagePoint p) {
		return original.pixelAt(p);
	}

	@Override
	public Pixel pixelAtUnchecked(int x, int y) {
		return original.pixelAtUnchecked(x, y);
	}

	@Override
	public MaskedImage windowOf(ImagePoint center, ImageSize size) {
		return new MaskedImage(original.windowOf(center, size), mask.windowOf(center, size));
	}

	public double getMean() {
		if (cachedMean == null) {
			cachedMean = Double.valueOf(calculateMean());
		}

		return cachedMean.doubleValue();
	}

	public double getStandardDeviation() {
		if (cachedStd == null) {
			cachedStd = Double.valueOf(calculateStandardDeviation());
		}

		return cachedStd.doubleValue();
	}

	/**
	 * Devolve o numero de pixels mascarados.
	 * 
	 * @return o numero de pixels mascarados
	 */
	public int getNumberOfMasked() {
		if (cachedMasked == null) {
			cachedMasked = Integer.valueOf(countMasked());
		}
		return cachedMasked;
	}

	/**
	 * Devolve {@code true} se o pixel na posicao dada for mascarado.
	 * 
	 * @param x a coordenada horizontal
	 * @param y a coordenada vertical
	 * @return {@code true} se o pixel na posicao dada for mascarado
	 */
	public boolean isMasked(ImagePoint p) {
		return mask.pixelAt(p).isBlack();
	}

	/**
	 * Conta o numero de pixels mascarados da imagem.
	 * 
	 * @return o numero de pixels mascarados da imagem 
	 */
	private int countMasked() {
		int masked = 0;

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {
				final ImagePoint p = ImagePoint.of(x, y);

				if (isMasked(p)) {
					masked++;
				}
			}
		}

		return masked;
	}

	/**
	 * Calcula a media da imagem levando em conta somente os pixels mascarados.
	 * 
	 * @return a media da imagem levando em conta somente os pixels mascarados
	 */
	private double calculateMean() {
		long sum = 0;

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {
				final ImagePoint p = ImagePoint.of(x, y);

				if (isMasked(p)) {
					sum += pixelAt(p).value();
				}
			}
		}

		return (double) sum / getNumberOfMasked();
	}

	/**
	 * Calcula o desvio padrao da imagem, levando em conta somente os pixels mascarados.
	 * 
	 * @return o desvio padrao da imagem, levando em conta somente os pixels mascarados
	 */
	private double calculateStandardDeviation() {
		final double mean = getMean();

		double sum = 0;

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {
				final ImagePoint p = ImagePoint.of(x, y);

				if (isMasked(p)) {
					final double diff = pixelAt(p).value() - mean;
					sum += diff * diff;
				}
			}
		}

		final double variance = sum / getNumberOfMasked();

		return Math.sqrt(variance);
	}
}
