package binarization.data;

import java.util.ArrayList;
import java.util.List;

/**
 * Representa um pixel de uma imagem. Definido somente para niveis de cinza.
 * Implementa o padrao <em>Flyweight</em> para compartilhar os pixels de mesmo
 * valor, externalizando sua posicao.
 *
 * @author denis
 * @see IImage
 * @see IBinaryImage
 * @see IGrayLevelImage
 */
public class Pixel {

	private static final int BLACK = 0;
	private static final int WHITE = 255;
	private static final List<Pixel> cache = initialize(WHITE + 1);
	//
	private final int value;

	private Pixel(int value) {
		this.value = value;
	}

	private static List<Pixel> initialize(int max) {
		final List<Pixel> values = new ArrayList<Pixel>(max);

		for (int i = 0; i < max; i++) {
			values.add(new Pixel(i));
		}

		return values;
	}

	/**
	 * Devolve o pixel correspondente ao valor. Deve ser nao-negativo, e
	 * seu valor deve estar abaixo do definido por {@code ofMaxGrayLevel}.
	 * Caso contrario, uma excecao sera lancada.
	 * 
	 * @param value o valor do qual se quer o pixel correspondente
	 * @return o pixel correspondente ao valor
	 * @throws IllegalArgumentException caso o valor esteja fora da faixa estipulada
	 */
	public static Pixel of(int value) {
		if (value < 0 || value > WHITE) {
			throw new IllegalArgumentException("Pixel value " + value + " out of range");
		}

		return cache.get(value);
	}

	/**
	 * Devolve o pixel correspondente ao valor, restringindo aos valores validos.
	 * De outra forma, ao inves de lancar uma excecao, devolve o pixel de valor
	 * valido mais proximo.
	 * 
	 * @param value o valor do qual se quer o pixel correspondente
	 * @return o pixel correspondente ao valor, restringindo aos valores validos
	 */
	public static Pixel ofRestricted(int value) {
		return cache.get(
				value < 0 ? 0
				: (value > WHITE ? WHITE
				: value));
	}

	/**
	 * Devolve o pixel correspondente ao preto para imagens binarias.
	 * 
	 * @return o pixel correspondente ao preto para imagens binarias
	 */
	public static Pixel ofBlack() {
		return cache.get(BLACK);
	}

	/**
	 * Devolve o pixel correspondente ao branco para imagens binarias.
	 * 
	 * @return o pixel correspondente ao branco para imagens binarias
	 */
	public static Pixel ofWhite() {
		return cache.get(WHITE);
	}

	/**
	 * Devolve o pixel correspondente ao valor maximo para imagens em niveis
	 * de cinza. O menor e implicitamente zero.
	 * 
	 * @return o pixel correspondente ao valor maximo para imagens em niveis de cinza
	 */
	public static Pixel ofMaxGrayLevel() {
		return cache.get(WHITE);
	}

	/**
	 * Devolve o valor deste pixel.
	 * 
	 * @return o valor deste pixel
	 */
	public int value() {
		return value;
	}

	/**
	 * Devolve {@code true} caso este pixel seja preto.
	 * 
	 * @return {@code true} caso este pixel seja preto
	 */
	public boolean isBlack() {
		return value == BLACK;
	}

	/**
	 * Devolve {@code true} caso este pixel seja branco.
	 * 
	 * @return {@code true} caso este pixel seja branco
	 */
	public boolean isWhite() {
		return value == WHITE;
	}

	@Override
	public String toString() {
		return "Pixel " + value;
	}
}
