package binarization.data;

/**
 * Implementacao simples de um construtor de imagem em niveis de cinza.
 * 
 * <p>Nao e imutavel, claro, mas sua mutabilidade se restringe a pintar os pixels
 * do objeto. Nao pode ser instanciado para forcar a utilizar os metodos de
 * fabrica na classe utilitaria {@link Images}</p>.
 *
 * @author denis
 */
class SimpleGrayLevelImageBuilder implements IGrayLevelImage.Builder {

	private final Pixel[][] pixels;
	private final ImageSize size;

	/**
	 * Constroi um construtor de imagem em niveis de cinza de tamanho dado .
	 * 
	 * @param size o tamanho da imagem a ser construida
	 * @throws IllegalArgumentException se a altura ou a largura forem negativas
	 */
	SimpleGrayLevelImageBuilder(ImageSize size) {
		if (size.isEmpty()) {
			throw new IllegalArgumentException("Size must be positive.");
		}
		this.pixels = initializePixels(size);
		this.size = size;
	}

	/**
	 * Inicializa os pixels como fundo.
	 * 
	 * @param size o tamanho da imagem
	 * @return os pixels inicializados como fundo
	 */
	private Pixel[][] initializePixels(ImageSize size) {
		final Pixel[][] background = new Pixel[size.width()][size.height()];

		for (int x = 0; x < size.width(); x++) {
			for (int y = 0; y < size.height(); y++) {
				background[x][y] = Pixel.ofWhite();
			}
		}

		return background;
	}

	@Override
	public void setPixel(ImagePoint p, Pixel pixel) {
		if (!p.isInside(size)) {
			throw new IndexOutOfBoundsException("Point p:" + p + " is not inside this builder.");
		}
		pixels[p.x()][p.y()] = pixel;
	}

	@Override
	public void setPixelUnchecked(int x, int y, Pixel pixel) {
		pixels[x][y] = pixel;
	}

	@Override
	public IGrayLevelImage build() {
		return new SimpleGrayLevelImage(pixels);
	}

	Pixel pixelAt(int x, int y) {
		return pixels[x][y];
	}
}
