package binarization.examples;

import binarization.IBinarizationMethod;
import binarization.IBinaryImageMetric;
import binarization.Main;
import binarization.Results;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Scanner;

/**
 * Ponto de entrada para utillizar os metodos de binarizacao e avalia-los ao mesmo tempo.
 * Produz as imagens binarizadas, a avaliacao de cada imagem e a media das avaliacoes,
 * alem do ranking obtido das avaliacoes.
 * 
 * <p>E verificado se o GT esqueleto sera utilizado na primeira linha da entrada,
 * assumindo que havera outros em cada outra linha do programa. Se nao houver GT
 * esqueleto ne primeira linha, somente as metricas que nao a usam serao utilizadas.
 * Caso ela esteja na primeira linha, mas nao haja GT esqueleto em algumas das
 * restantes, o valor das metricas que utilizam a GT esqueleto e indefinido.</p>
 *
 * @author denis
 */
public class ExampleAllNoReflection {

	private static final String FILE_SEPARATOR = System.getProperty("file.separator");
	private static final String CURRENT_TIME = new SimpleDateFormat("dd.MM.yyyy_kk.mm.ss").format(Calendar.getInstance().getTime());
	private static final String RESULTS_DIRNAME = "bin_results_";
	private static final String AVERAGES_DIRNAME = "averages";

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private ExampleAllNoReflection() {
	}

	/**
	 * @param args args[0] possui o arquivo de entrada com as imagens
	 */
	public static void main(String[] args) {
		if (args.length < 1) {
			System.out.println("Requer um arquivo com as imagens de entrada.");
			System.exit(-1);
		}
		final String filename = args[0];

		System.out.println("Entrada: " + filename);
		System.out.println();

		final Collection<IBinarizationMethod> methods = Methods.getAllMethods();
		final Collection<IBinaryImageMetric> metrics = hasSkeletonizedGroundTruth(args[0])
				? Metrics.getAllMetricsWithPFM()
				: Metrics.getAllMetrics();

		final String resultsPath = RESULTS_DIRNAME + Main.extractName(filename) + "_" + CURRENT_TIME + FILE_SEPARATOR;
		final String averagesPath = resultsPath + AVERAGES_DIRNAME + FILE_SEPARATOR;

		Scanner input = null;
		try {
			input = new Scanner(new File(filename));
			final Results results = Main.binarizeAndEvaluate(input, resultsPath, methods, metrics);
			Main.printResults(filename, averagesPath, results, methods, metrics);

			for (Main.RankingType type : Main.RankingType.values()) {
				Main.printRanking(filename, resultsPath, results, methods, metrics, type);
			}

		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} finally {
			if (input != null) {
				input.close();
			}
		}
	}

	/**
	 * Verifica se ha GT esqueleto na primeira linha da entrada.
	 * 
	 * @param filename o arquivo de entrada
	 * @return {@code true} se houver GT esqueleto na primeira linha da entrada
	 */
	private static boolean hasSkeletonizedGroundTruth(String filename) {
		boolean hasSkeletonized = false;
		Scanner scanner = null;
		try {
			scanner = new Scanner(new File(filename));
			final Scanner line = new Scanner(scanner.nextLine());
			line.next();
			line.next();
			hasSkeletonized = line.hasNext();

		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} finally {
			if (scanner != null) {
				scanner.close();
			}
		}

		return hasSkeletonized;
	}
}