package binarization.methods;

import binarization.IBinarizationMethod;
import binarization.data.IBinaryImage;
import binarization.data.IGrayLevelImage;
import binarization.data.Images;
import binarization.data.ImageSize;
import binarization.data.ImagePoint;

/**
 * Metodo local adaptativo que calcula o limiar para uma janela em torno de
 * cada pixel para determinar o limiar daquele pixel.
 *
 * @author denis
 * @see IBinarizationMethod
 * @see IBinaryImage
 * @see IGrayLevelImage
 * @see AbstractGlobalThresholding
 */
public abstract class AbstractLocalAdaptiveThresholding implements IBinarizationMethod {

	private final int windowSize;

	/**
	 * Constroi um metodo local de tamanho de janela dado.
	 * 
	 * @param windowSize o tamanho da janela quadrada
	 * @throws IllegalArgumentException caso o tamanho seja nao-positivo
	 */
	public AbstractLocalAdaptiveThresholding(int windowSize) {
		this.windowSize = windowSize;

		if (this.windowSize <= 0) {
			throw new IllegalArgumentException("Window size must be positive.");
		}
	}

	/**
	 * Devolve o limiar calculado pelo metodo para a janela dada.
	 * 
	 * @param window a ajanela da imagem em niveis de cinza
	 * @return o limiar calculado pelo metodo para a janela dada
	 */
	abstract public double getThreshold(IGrayLevelImage window);

	@Override
	abstract public String getLabel();

	@Override
	public final IBinaryImage binarize(IGrayLevelImage input) {
		final IBinaryImage.Builder builder = Images.newBinaryImageBuilder(input.size());

		for (int x = 0; x < input.size().width(); x++) {
			for (int y = 0; y < input.size().height(); y++) {
				final ImagePoint p = ImagePoint.of(x, y);

				final IGrayLevelImage window = input.windowOf(p, ImageSize.of(windowSize));

				final double threshold = getThreshold(window);
				final int intensity = input.pixelAt(p).value();

				if (intensity <= threshold) {
					builder.setPixel(p);
				}
			}
		}

		return builder.build();
	}

	public int getWindowSize() {
		return windowSize;
	}
}
