package binarization.methods;

import binarization.IBinarizationMethod;
import binarization.data.IGrayLevelImage;

/**
 * Metodo de Sauvola. (precisa de citacao)
 * 
 * <p>Calcula o limiar para uma janela igual a 
 * {@code m(x, y) * (1 + k * ((s(x, y) / R) - 1))}, onde:<ol>
 * 
 * <li>{@code k} e o peso do desvio padrao,</li>
 * <li>{@code R} e a faixa dinamica do desvio padrao,</li>
 * <li>{@code m(x, y) e s(x, y)} sao a media e o desvio padrao da janela
 * centrada no pixel {@code (x, y)}, respectivamente.</li>
 * </ol></p>
 * 
 * <p>Os valorer recomendados sao: {@code k = 0.5, R = 128}.</p>
 * 
 * <p>Este metodo e uma adaptacao do metodo de Niblack ({@link NiblackMethod}).</p>
 * 
 * <p>O metodo de Sauvola tambem estipula a binarizacao de elementos nao-textuais.
 * Esta parte nao e utilizada.</p>
 *
 * @author denis
 * @see NiblackMethod
 */
public class SauvolaMethod extends AbstractLocalAdaptiveThresholding implements IBinarizationMethod {

	private static final double RECOMMENDED_WEIGHT = 0.5;
	private static final double RECOMMENDED_RANGE = 128;
//
	private final double weight;
	private final double range;

	/**
	 * Metodo de Sauvola com tamanho de janela, peso e faixa dados.
	 * 
	 * @param windowSize o tamnho da janela quadrada
	 * @param weight o peso de desvio padrao k
	 * @param range a faixa dinamica do desvio padrao R
	 */
	public SauvolaMethod(int windowSize, double weight, double range) {
		super(windowSize);
		this.weight = weight;
		this.range = range;
	}

	/**
	 * Metodo de Sauvola com tamanho de janela e peso dados e faixa recomendada.
	 * 
	 * @param windowSize o tamnho da janela quadrada
	 * @param weight o peso de desvio padrao k
	 */
	public SauvolaMethod(int windowSize, double weight) {
		this(windowSize, weight, RECOMMENDED_RANGE);
	}

	/**
	 * Metodo de Sauvola com tamanho de janela dada e peso e faixa recomendados.
	 * 
	 * @param windowSize o tamnho da janela quadrada
	 */
	public SauvolaMethod(int windowSize) {
		this(windowSize, RECOMMENDED_WEIGHT, RECOMMENDED_RANGE);
	}

	@Override
	public double getThreshold(IGrayLevelImage window) {
		final double mean = window.getMean();
		final double std = window.getStandardDeviation();

		return mean * (1.0 + weight * ((std / range) - 1));
	}

	@Override
	public String getLabel() {
		return "sauvola_window" + getWindowSize() + "_k" + weight + "_r" + range;
	}

	@Override
	public String toString() {
		return "Sauvola, Window Size: " + getWindowSize() + ", k: " + weight + ", R: " + range;
	}
}
