package binarization.metrics;

import binarization.IBinaryImageMetric;
import binarization.data.IBinaryImage;
import binarization.util.Comparators;
import java.text.Format;
import java.text.NumberFormat;
import java.util.Comparator;

/**
 * Peak Signal to Noise Ratio, ou PSNR, mede o quao parecida sao as imagens.
 * 
 * <p>Esta metrica avalia a imagem binarizada da seguinte forma: <ol>
 * 
 * <li>Para cada pixel, calcula a diferenca entre o GT e a imagem binarizada,
 *     elevando ao quadrado, chamando-o de {@code diff(x, y) ^ 2}. </li>
 * <li>Faz o somatorio de todos {@code diff(x, y) ^ 2} e divide por
 *     {@code comprimento * altura}. Este valor sera chamado de {@code mse}. </li>
 * <li>O valor final e {@code PSNR = 10 * log(c ^ 2 / mse)}, no qual
 *     {@code c} e a diferenca entre o <em>background e o foreground</em>. </li>
 * </ol></p>
 * 
 * <p>Esta implementacao, no entanto, faz a seguinte simplificacao: <ol>
 * 
 * <li>Note que {@code diff(x, y) = 0}, se for true positive ou negative, e
 *              {@code diff(x, y) = c}, se for false positive ou negative.</li>
 * <li>Alem disso, {@code numero de pixels = comprimento * altura}, e
 *                 {@code numero de pixels = tp + tn + fp + fn},
 *      ja que cada pixel esta em uma das classificacoes.</li>
 * <li>Logo, {@code mse = (fp + fn) * (c ^ 2) / (tp + tn + fp + fn)}. </li>
 * <li>Portanto, {@code PSNR = 10 * log((tp + tn + fp + fn) / (fp + fn))},
 *      pois {@code c ^ 2} aparece no numerador e no denominador. </li>
 * </ol></p>
 * 
 * <p>Quanto mais alto for o valor devolvido, mais parecida com o ground truth
 * sera a imagem binarizada.</p>
 * 
 * <p><em>Nota da implementacao: e um enum para garantir o padrao singleton, nao e
 * {@code static} para utilizar polimorfismo, e nao sao todos um unico enum
 * para poder ser extensivel e mais organizado.</em></p>
 *
 * @author denis
 */
public enum PeakSignalToNoiseRatio implements IBinaryImageMetric {

	/**Unica instancia da classe*/
	INSTANCE;

	@Override
	public final double evaluate(IBinaryImage groundTruth, IBinaryImage binarized) {
		final BinaryImageAccumulator acc = new BinaryImageAccumulator(groundTruth, binarized);

		final long tp = acc.getTruePositives();
		final long tn = acc.getTrueNegatives();
		final long fp = acc.getFalsePositives();
		final long fn = acc.getFalseNegatives();

		final double numerator = tp + tn + fp + fn;
		final double denominator = fp + fn + Double.MIN_VALUE;

		return 10.0 * Math.log10(numerator / denominator);
	}

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized, IBinaryImage skeletonizedGT) {
		//nao utiliza a terceira imagem
		return evaluate(groundTruth, binarized);
	}

	@Override
	public String getLabel() {
		return "PSNR";
	}

	@Override
	public Format getFormat() {
		//NumberFormat e mutavel, portanto necessita de copia defensiva
		return NumberFormat.getInstance();
	}

	@Override
	public Comparator<Double> getResultComparator() {
		return Comparators.INCREASING;
	}

	@Override
	public String toString() {
		return "Peak Signal to Noise Ratio";
	}
}
