package binarization.metrics;

import binarization.IBinaryImageMetric;
import binarization.data.IBinaryImage;
import java.text.Format;
import java.util.Comparator;

/**
 * Pseudo F-Measure, obtida do Precision e pseudo-Recall 
 * (Recall do <em>skeletonized ground truth</em>).
 * 
 * <p>Esta metrica avalia a imagem binarizada da seguinte forma: <ol>
 * 
 * <li>Calcula o valor de {@code p-recall = tps / (tps + fns)} </li>
 * <li>Calcula o valor de {@code precision = tp / (tp + fp)} </li>
 * <li>O valor final e {@code pF-Measure = 2*p-recall*precision / (p-recall + precision)} </li>
 * </ol></p>
 * 
 * <p>tp e fp sao o numero de true positives e false positives, respectivamente,
 * em relacao ao <em>ground truth</em>.</p>
 * 
 * <p>tps e fns sao o numero de true positives e false negatives, respectivamente,
 * em relacao ao <em>skeletonized ground truth</em>, i.e., a silhueta de um
 * pixel de largura do <em>ground truth</em>, que deve ser dado juntamente a ele.</p>
 * 
 * <p>O valor devolvido esta no intervalo [0, 1], e quanto mais alto for,
 * mais parecida com o ground truth sera a imagem binarizada.</p>
 * 
 * <p><strong>Nao implementa o contrato basico de avaliacao de uma metrica, pois
 * necessita do <em>skeletonized ground truth</em> para calcular o pseudo-recall.
 * Por isso, fere o principio de substituicao de Liskov. Talvez uma possivel
 * re-estruturacao resolva esta peculiaridade. Esta metrica so implementa a 
 * infterface de metricas por definir os outros metodos fora os de avaliacao.
 * </strong></p>
 * 
 * <p><em>Nota da implementacao: e um enum para garantir o padrao singleton, nao e
 * {@code static} para utilizar polimorfismo, e nao sao todos um unico enum
 * para poder ser extensivel e mais organizado.</em></p>
 *
 * @author denis
 * @see FMeasure
 */
public enum PseudoFMeasure implements IBinaryImageMetric {

	/**Unica instancia da classe*/
	INSTANCE;

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized) {
		throw new UnsupportedOperationException("This evaluation needs the skeletonized GT.");
	}

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized, IBinaryImage skeletonizedGT) {
		if (skeletonizedGT.isEmpty()) {
			return 0.0;
		}

		final BinaryImageAccumulator acc = new BinaryImageAccumulator(groundTruth, binarized);
		final BinaryImageAccumulator accSkeletonized = new BinaryImageAccumulator(skeletonizedGT, binarized);

		final long tp = acc.getTruePositives();
		final long fp = acc.getFalsePositives();

		final long tps = accSkeletonized.getTruePositives();
		final long fns = accSkeletonized.getFalseNegatives();

		final double pseudoRecall = FMeasureHelper.calculateRecall(tps, fns);
		final double precision = FMeasureHelper.calculatePrecision(tp, fp);

		return FMeasureHelper.calculateFMeasure(pseudoRecall, precision);
	}

	@Override
	public String getLabel() {
		return "pFM";
	}

	@Override
	public Format getFormat() {
		return FMeasureHelper.getFormat();
	}

	@Override
	public Comparator<Double> getResultComparator() {
		return FMeasureHelper.getResultComparator();
	}

	@Override
	public String toString() {
		return "Pseudo F-Measure";
	}
}
